@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-embed.h,v 1.27 2004/01/04 12:52:04 chpe Exp $
+ *  $Id: ephy-embed.h,v 1.41 2004/06/25 19:13:39 marco Exp $
  */
 
 #ifndef EPHY_EMBED_H
@@ -32,13 +32,15 @@ G_BEGIN_DECLS
 
 #define EPHY_TYPE_EMBED			(ephy_embed_get_type ())
 #define EPHY_EMBED(o)			(G_TYPE_CHECK_INSTANCE_CAST ((o), EPHY_TYPE_EMBED, EphyEmbed))
-#define EPHY_EMBED_IFACE(k)		(G_TYPE_CHECK_CLASS_CAST((k), EPHY_TYPE_EMBED, EphyEmbedIFace))
+#define EPHY_EMBED_IFACE(k)		(G_TYPE_CHECK_CLASS_CAST((k), EPHY_TYPE_EMBED, EphyEmbedIface))
 #define EPHY_IS_EMBED(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), EPHY_TYPE_EMBED))
 #define EPHY_IS_EMBED_IFACE(k)		(G_TYPE_CHECK_CLASS_TYPE ((k), EPHY_TYPE_EMBED))
-#define EPHY_EMBED_GET_IFACE(inst)	(G_TYPE_INSTANCE_GET_INTERFACE ((inst), EPHY_TYPE_EMBED, EphyEmbedIFace))
+#define EPHY_EMBED_GET_IFACE(inst)	(G_TYPE_INSTANCE_GET_INTERFACE ((inst), EPHY_TYPE_EMBED, EphyEmbedIface))
 
-typedef struct EphyEmbed	EphyEmbed;
-typedef struct EphyEmbedIFace	EphyEmbedIFace;
+#define EPHY_TYPE_EMBED_CHROME_MASK     (ephy_embed_chrome_get_type ())
+
+typedef struct _EphyEmbed	EphyEmbed;
+typedef struct _EphyEmbedIface	EphyEmbedIface;
 
 typedef enum
 {
@@ -57,65 +59,16 @@ typedef enum
 
 typedef enum
 {
-	EMBED_CHROME_NONE			= 0,
-	EMBED_CHROME_DEFAULT			= 1 << 0,
-	EMBED_CHROME_MENUBARON			= 1 << 1,
-	EMBED_CHROME_TOOLBARON			= 1 << 2,
-	EMBED_CHROME_BOOKMARKSBARON		= 1 << 3,
-	EMBED_CHROME_BOOKMARKSBAR_DEFAULT	= 1 << 4,
-	EMBED_CHROME_STATUSBARON		= 1 << 5,
-	EMBED_CHROME_WINDOWRAISED		= 1 << 6,
-	EMBED_CHROME_WINDOWLOWERED		= 1 << 7,
-	EMBED_CHROME_CENTERSCREEN		= 1 << 8,
-	EMBED_CHROME_OPENASDIALOG		= 1 << 9,
-	EMBED_CHROME_OPENASCHROME		= 1 << 10,
-	EMBED_CHROME_OPENASPOPUP		= 1 << 11,
-	EMBED_CHROME_PPVIEWTOOLBARON		= 1 << 12
-} EmbedChromeMask;
+	EPHY_EMBED_CHROME_MENUBAR	= 1 << 0,
+	EPHY_EMBED_CHROME_TOOLBAR	= 1 << 1,
+	EPHY_EMBED_CHROME_STATUSBAR	= 1 << 2,
+	EPHY_EMBED_CHROME_BOOKMARKSBAR	= 1 << 3
+} EphyEmbedChrome;
 
-typedef enum
-{
-	EMBED_RELOAD_NORMAL	 = 1 << 1,
-	EMBED_RELOAD_BYPASSCACHE = 1 << 2,
-	EMBED_RELOAD_BYPASSPROXY = 1 << 3
-} EmbedReloadFlags;
-
-typedef struct
-{
-	gboolean print_to_file;
-	char *printer;
-	char *file;
-	char *paper;
-	int top_margin;
-	int bottom_margin;
-	int left_margin;
-	int right_margin;
-	int pages;
-	int from_page;
-	int to_page;
-	int frame_type;
-	int orientation;
-	gboolean print_color;
-	gboolean preview;
-
-	/*
-	 * &T - title
-	 * &U - Document URL
-	 * &D - Date/Time
-	 * &P - Page Number
-	 * &PT - Page Number with total Number of Pages (example: 1 of 34)
-	 *
-	 * So, if headerLeftStr = "&T" the title and the document URL
-	 * will be printed out on the top left-hand side of each page.
-	 */
-	char *header_left_string;
-	char *header_center_string;
-	char *header_right_string;
-	char *footer_left_string;
-	char *footer_center_string;
-	char *footer_right_string;
-}
-EmbedPrintInfo;
+#define EPHY_EMBED_CHROME_ALL (EPHY_EMBED_CHROME_MENUBAR |	\
+			       EPHY_EMBED_CHROME_TOOLBAR |	\
+			       EPHY_EMBED_CHROME_STATUSBAR |	\
+			       EPHY_EMBED_CHROME_BOOKMARKSBAR)
 
 typedef enum
 {
@@ -136,7 +89,7 @@ typedef enum
 	STATE_IS_SECURE_HIGH
 } EmbedSecurityLevel;
 
-struct EphyEmbedIFace
+struct _EphyEmbedIface
 {
 	GTypeInterface base_iface;
 
@@ -165,23 +118,27 @@ struct EphyEmbedIFace
 					 EmbedState state);
 	void	 (* new_window)		(EphyEmbed *embed,
 					 EphyEmbed **new_embed,
-					 EmbedChromeMask chromemask);
+					 EphyEmbedChrome chromemask);
 	gboolean (* dom_mouse_click)	(EphyEmbed *embed,
 					 EphyEmbedEvent *event);
 	gboolean (* dom_mouse_down)	(EphyEmbed *embed,
 					 EphyEmbedEvent *event);
-	void	 (* popup_blocked)	(EphyEmbed *embed);
+	void	 (* popup_blocked)	(EphyEmbed *embed,
+					 const char *address,
+					 const char *features);
 	void	 (* security_change)	(EphyEmbed *embed,
 					 EmbedSecurityLevel level);
 	void	 (* zoom_change)	(EphyEmbed *embed,
 					 float new_zoom);
+	void	 (* content_change)	(EphyEmbed *embed,
+					 const char *uri);
 
 	/* Methods  */
 	void		   (* load_url)			(EphyEmbed *embed,
 							 const char *url);
 	void		   (* stop_load)		(EphyEmbed *embed);
 	void		   (* reload)			(EphyEmbed *embed,
-							 EmbedReloadFlags flags);
+							 gboolean force);
 	gboolean	   (* can_go_back)		(EphyEmbed *embed);
 	gboolean	   (* can_go_forward)		(EphyEmbed *embed);
 	gboolean	   (* can_go_up)		(EphyEmbed *embed);
@@ -207,22 +164,22 @@ struct EphyEmbedIFace
 	void		   (* get_security_level)	(EphyEmbed *embed,
 						  	 EmbedSecurityLevel *level,
 						  	 char **description);
-	void		   (* zoom_set)			(EphyEmbed *embed,
-							 float zoom,
-							 gboolean reflow);
-	float		   (* zoom_get)			(EphyEmbed *embed);
+	void		   (* set_zoom)			(EphyEmbed *embed,
+							 float zoom);
+	float		   (* get_zoom)			(EphyEmbed *embed);
 	void		   (* find_set_properties)	(EphyEmbed *embed,
-							 char *search_string,
+							 const char *search_string,
 							 gboolean case_sensitive,
 							 gboolean wrap_around);
 	gboolean	   (* find_next)		(EphyEmbed *embed,
 							 gboolean backwards);
+	char *		   (* get_encoding)		(EphyEmbed *embed);
+	gboolean	   (* has_automatic_encoding)	(EphyEmbed *embed);
 	void		   (* set_encoding)		(EphyEmbed *embed,
 							 const char *encoding);
-	EphyEncodingInfo * (* get_encoding_info)	(EphyEmbed *embed);
-	void		   (* print)			(EphyEmbed *embed,
-							 EmbedPrintInfo *info);
-	void		   (* print_preview_close)	(EphyEmbed *embed);
+	void		   (* print)			(EphyEmbed *embed);
+	void		   (* set_print_preview_mode)	(EphyEmbed *embed,
+							 gboolean mode);
 	int		   (* print_preview_n_pages)	(EphyEmbed *embed);
 	void		   (* print_preview_navigate)	(EphyEmbed *embed,
 							 EmbedPrintPreviewNavType type,
@@ -231,6 +188,8 @@ struct EphyEmbedIFace
 	gboolean	   (* has_modified_forms)	(EphyEmbed *embed);
 };
 
+GType		  ephy_embed_chrome_get_type		(void);
+
 GType		  ephy_embed_get_type			(void);
 
 /* Base */
@@ -240,7 +199,7 @@ void		  ephy_embed_load_url			(EphyEmbed *embed,
 void		  ephy_embed_stop_load			(EphyEmbed *embed);
 
 void		  ephy_embed_reload			(EphyEmbed *embed,
-							 EmbedReloadFlags flags);
+							 gboolean force);
 
 char 		 *ephy_embed_get_title			(EphyEmbed *embed);
 
@@ -281,18 +240,17 @@ void		  ephy_embed_shistory_go_nth		(EphyEmbed *embed,
 
 void		  ephy_embed_get_security_level		(EphyEmbed *embed,
 							 EmbedSecurityLevel *level,
-						 	char **description);
+						 	 char **description);
 
 /* Zoom */
-void		  ephy_embed_zoom_set			(EphyEmbed *embed,
-							 float zoom,
-							 gboolean reflow);
+void		  ephy_embed_set_zoom			(EphyEmbed *embed,
+							 float zoom);
 
-float		  ephy_embed_zoom_get			(EphyEmbed *embed);
+float		  ephy_embed_get_zoom			(EphyEmbed *embed);
 
 /* Find */
 void		  ephy_embed_find_set_properties	(EphyEmbed *embed,
-							 char *search_string,
+							 const char *search_string,
 							 gboolean case_sensitive,
 							 gboolean wrap_around);
 
@@ -300,16 +258,18 @@ gboolean	  ephy_embed_find_next			(EphyEmbed *embed,
 							 gboolean backwards);
 
 /* Encoding */
+char		 *ephy_embed_get_encoding		(EphyEmbed *embed);
+
+gboolean	  ephy_embed_has_automatic_encoding	(EphyEmbed *embed);
+
 void		  ephy_embed_set_encoding		(EphyEmbed *embed,
 							 const char *encoding);
 
-EphyEncodingInfo *ephy_embed_get_encoding_info		(EphyEmbed *embed);
-
 /* Print */
-void		  ephy_embed_print			(EphyEmbed *embed,
-							 EmbedPrintInfo *info);
+void		  ephy_embed_print			(EphyEmbed *embed);
 
-void		  ephy_embed_print_preview_close	(EphyEmbed *embed);
+void		  ephy_embed_set_print_preview_mode	(EphyEmbed *embed,
+							 gboolean preview_mode);
 
 int		  ephy_embed_print_preview_n_pages	(EphyEmbed *embed);
 
