@@ -27,12 +27,12 @@ G_BEGIN_DECLS
 
 typedef struct EggToolbarsModelClass EggToolbarsModelClass;
 
-#define EGG_TOOLBARS_MODEL_TYPE             (egg_toolbars_model_get_type ())
-#define EGG_TOOLBARS_MODEL(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), EGG_TOOLBARS_MODEL_TYPE, EggToolbarsModel))
-#define EGG_TOOLBARS_MODEL_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), EGG_TOOLBARS_MODEL_TYPE, EggToolbarsModelClass))
-#define IS_EGG_TOOLBARS_MODEL(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), EGG_TOOLBARS_MODEL_TYPE))
-#define IS_EGG_TOOLBARS_MODEL_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), EGG_TOOLBARS_MODEL_TYPE))
-#define EGG_TOOLBARS_MODEL_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), EGG_TOOLBARS_MODEL_TYPE, EggToolbarsModelClass))
+#define EGG_TYPE_TOOLBARS_MODEL             (egg_toolbars_model_get_type ())
+#define EGG_TOOLBARS_MODEL(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), EGG_TYPE_TOOLBARS_MODEL, EggToolbarsModel))
+#define EGG_TOOLBARS_MODEL_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), EGG_TYPE_TOOLBARS_MODEL, EggToolbarsModelClass))
+#define EGG_IS_TOOLBARS_MODEL(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), EGG_TYPE_TOOLBARS_MODEL))
+#define EGG_IS_TOOLBARS_MODEL_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), EGG_TYPE_TOOLBARS_MODEL))
+#define EGG_TOOLBARS_MODEL_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), EGG_TYPE_TOOLBARS_MODEL, EggToolbarsModelClass))
 
 typedef struct EggToolbarsModel EggToolbarsModel;
 typedef struct EggToolbarsModelPrivate EggToolbarsModelPrivate;
@@ -41,13 +41,19 @@ typedef struct EggToolbarsModelPrivate EggToolbarsModelPrivate;
 
 typedef enum
 {
-  EGG_TB_MODEL_NOT_REMOVABLE = 1,
-  EGG_TB_MODEL_ICONS_ONLY = 2
+  EGG_TB_MODEL_NOT_REMOVABLE	 = 1 << 0,
+  EGG_TB_MODEL_ICONS_ONLY	 = 1 << 1,
+  EGG_TB_MODEL_TEXT_ONLY	 = 1 << 2,
+  EGG_TB_MODEL_ICONS_TEXT	 = 1 << 3,
+  EGG_TB_MODEL_ICONS_TEXT_HORIZ	 = 1 << 4,
+  EGG_TB_MODEL_ACCEPT_ITEMS_ONLY = 1 << 5
 } EggTbModelFlags;
 
 struct EggToolbarsModel
 {
   GObject parent_object;
+
+  /*< private >*/
   EggToolbarsModelPrivate *priv;
 };
 
@@ -68,6 +74,14 @@ struct EggToolbarsModelClass
 			    int position);
   void (* toolbar_removed) (EggToolbarsModel *group,
 			    int position);
+  char * (* get_item_type) (EggToolbarsModel *model,
+			    GdkAtom           dnd_type);
+  char * (* get_item_id)   (EggToolbarsModel *model,
+			    const char       *type,
+			    const char       *data);
+  char * (* get_item_data) (EggToolbarsModel *model,
+			    const char       *type,
+			    const char       *id);
 
   /* Virtual Table */
   gboolean (* add_item)    (EggToolbarsModel *t,
@@ -75,19 +89,11 @@ struct EggToolbarsModelClass
 			    int               position,
 			    const char       *id,
 			    const char       *type);
-  char * (* get_item_type) (EggToolbarsModel *t,
-			    GdkAtom           dnd_type);
-  char * (* get_item_id)   (EggToolbarsModel *t,
-			    const char       *type,
-			    const char       *name);
-  char * (* get_item_name) (EggToolbarsModel *t,
-			    const char       *type,
-			    const char       *id);
 };
 
 GType		  egg_toolbars_model_get_type       (void);
 EggToolbarsModel *egg_toolbars_model_new	    (void);
-void              egg_toolbars_model_load           (EggToolbarsModel *t,
+gboolean          egg_toolbars_model_load           (EggToolbarsModel *t,
 						     const char *xml_file);
 void              egg_toolbars_model_save           (EggToolbarsModel *t,
 						     const char *xml_file,
@@ -108,7 +114,7 @@ char             *egg_toolbars_model_get_item_type  (EggToolbarsModel *t,
 char             *egg_toolbars_model_get_item_id    (EggToolbarsModel *t,
 						     const char       *type,
 			                             const char       *name);
-char             *egg_toolbars_model_get_item_name  (EggToolbarsModel *t,
+char             *egg_toolbars_model_get_item_data  (EggToolbarsModel *t,
 						     const char       *type,
 			                             const char       *id);
 gboolean	  egg_toolbars_model_add_item       (EggToolbarsModel *t,
@@ -123,10 +129,12 @@ void		  egg_toolbars_model_remove_item    (EggToolbarsModel *t,
 						     int               position);
 int		  egg_toolbars_model_n_items	    (EggToolbarsModel *t,
 						     int               toolbar_position);
-const char	 *egg_toolbars_model_item_nth	    (EggToolbarsModel *t,
+void	 	  egg_toolbars_model_item_nth	    (EggToolbarsModel *t,
 						     int	       toolbar_position,
 						     int               position,
-						     gboolean         *is_separator);
+						     gboolean         *is_separator,
+						     const char      **id,
+						     const char      **type);
 int		  egg_toolbars_model_n_toolbars	    (EggToolbarsModel *t);
 const char	 *egg_toolbars_model_toolbar_nth    (EggToolbarsModel *t,
 						     int               position);
