@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-node-view.h,v 1.5 2003/07/12 16:25:07 xan Exp $
+ *  $Id: ephy-node-view.h,v 1.8 2004/03/03 19:54:35 chpe Exp $
  */
 
 #ifndef __EPHY_NODE_VIEW_H
@@ -42,6 +42,7 @@ typedef struct
 {
 	GtkTreeView parent;
 
+	/*< private >*/
 	EphyNodeViewPrivate *priv;
 } EphyNodeView;
 
@@ -66,7 +67,7 @@ typedef struct
 
 	void (*node_activated)      (EphyNodeView *view, EphyNode *node);
 	void (*node_selected)       (EphyNodeView *view, EphyNode *node);
-	void (*node_dropped)        (EphyNodeView *view, EphyNode *node, GList *nodes);
+	void (*node_dropped)        (EphyNodeView *view, EphyNode *node, GList *uris);
 	void (*show_popup)          (EphyNodeView *view);
 } EphyNodeViewClass;
 
@@ -85,6 +86,12 @@ GtkTreeViewColumn *ephy_node_view_add_column	      (EphyNodeView *view,
 						       EphyNodeViewFlags flags,
 						       EphyTreeModelNodeValueFunc icon_func);
 
+int		   ephy_node_view_add_data_column     (EphyNodeView *view,
+			                               GType value_type,
+						       int prop_id,
+			                               EphyTreeModelNodeValueFunc func,
+						       gpointer data);
+
 void		   ephy_node_view_remove              (EphyNodeView *view);
 
 gboolean           ephy_node_view_has_selection       (EphyNodeView *view,
@@ -98,7 +105,7 @@ void	           ephy_node_view_select_node         (EphyNodeView *view,
 void	           ephy_node_view_enable_drag_source  (EphyNodeView *view,
 						       GtkTargetEntry *types,
 						       int n_types,
-						       guint prop_id);
+						       int column_id);
 
 void	           ephy_node_view_enable_drag_dest    (EphyNodeView *view,
 						       GtkTargetEntry *types,
