@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks-menu.c,v 1.22 2004/02/11 14:38:55 marco Exp $
+ *  $Id: ephy-bookmarks-menu.c,v 1.27 2004/07/02 09:03:42 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -92,6 +92,8 @@ ephy_bookmarks_menu_clean (EphyBookmarksMenu *menu)
 {
 	EphyBookmarksMenuPrivate *p = menu->priv;
 
+	START_PROFILER ("Cleaning bookmarks menu")
+
 	if (p->ui_id > 0)
 	{
 		gtk_ui_manager_remove_ui (p->merge, p->ui_id);
@@ -104,10 +106,12 @@ ephy_bookmarks_menu_clean (EphyBookmarksMenu *menu)
 		gtk_ui_manager_remove_action_group (p->merge, p->action_group);
 		g_object_unref (p->action_group);
 	}
+
+	STOP_PROFILER ("Cleaning bookmarks menu")
 }
 
 static void
-go_location_cb (GtkAction *action, char *location, EphyWindow *window)
+open_bookmark_cb (GtkAction *action, char *location, EphyWindow *window)
 {
 	ephy_window_load_url (window, location);
 }
@@ -179,7 +183,7 @@ sort_bookmarks (gconstpointer a, gconstpointer b)
 }
 
 static void
-add_bookmarks_menu (EphyBookmarksMenu *menu, EphyNode *node, const char *path)
+create_menu (EphyBookmarksMenu *menu, EphyNode *node, const char *path)
 {
 	GPtrArray *children;
 	EphyBookmarksMenuPrivate *p = menu->priv;
@@ -201,18 +205,22 @@ add_bookmarks_menu (EphyBookmarksMenu *menu, EphyNode *node, const char *path)
 			GtkAction *action;
 			EphyNode *child;
 			long id;
-			char verb[30], name[30];
+			char verb[30], name[30], accel_path[60];
 
 			child = l->data;
 			id = ephy_node_get_id (child);
-			g_sprintf (verb, "OpenTopic%ld", ephy_node_get_id (child));
-			g_sprintf (name, "%sName", verb);
+			g_snprintf (verb, sizeof (verb), 
+				    "OpenBmk%d", ephy_node_get_id (child));
+			g_snprintf (name, sizeof (name), "%sName", verb);
+			g_snprintf (accel_path, sizeof (accel_path),
+				    "<Actions>/BookmarksActions/%s", verb);
 
 			action = ephy_bookmark_action_new (verb, id);
+			gtk_action_set_accel_path (action, accel_path);
 			gtk_action_group_add_action (p->action_group, action);
 			g_object_unref (action);
-			g_signal_connect (action, "go_location",
-					  G_CALLBACK (go_location_cb), p->window);
+			g_signal_connect (action, "open",
+					  G_CALLBACK (open_bookmark_cb), p->window);
 
 			gtk_ui_manager_add_ui (p->merge, p->ui_id, path,
 					       name, verb,
@@ -222,6 +230,83 @@ add_bookmarks_menu (EphyBookmarksMenu *menu, EphyNode *node, const char *path)
 	}
 }
 
+static void
+ensure_folder (EphyBookmarksMenu *menu, const char *path, const char *folder)
+{
+	EphyBookmarksMenuPrivate *p = menu->priv;
+	char *action_name;
+	GtkAction *action;
+
+	g_return_if_fail (folder != NULL);
+
+	action_name = g_strdup_printf ("TopicAction-%s\n", folder);
+	action = gtk_action_group_get_action (p->action_group, action_name);
+
+	if (action == NULL)
+	{
+		char *full_path;
+
+		action = g_object_new (GTK_TYPE_ACTION,
+				       "name", action_name,
+				       "label", folder,
+				       "hide_if_empty", FALSE,
+				       NULL);
+		gtk_action_group_add_action (p->action_group, action);
+		g_object_unref (action);
+
+		full_path = g_strdup_printf (BOOKMARKS_MENU_PATH "/%s", path);
+		gtk_ui_manager_add_ui (p->merge, p->ui_id, full_path,
+				       folder, action_name,
+				       GTK_UI_MANAGER_MENU, FALSE);
+
+		g_free (full_path);
+	}
+
+	g_free (action_name);
+}
+
+static char *
+create_submenu (EphyBookmarksMenu *menu, EphyNode *topic)
+{
+	const char *tmp;
+	char *title, *path, *p, *p2;
+
+	tmp = ephy_node_get_property_string (topic, EPHY_NODE_KEYWORD_PROP_NAME);
+	title = ephy_string_double_underscores (tmp);
+
+	g_return_val_if_fail (title != NULL, NULL);
+
+	p = p2 = title;
+	while (p)
+	{
+		if (*p == '\0' || g_utf8_get_char (p) == '/')
+		{
+			char *folder, *path;
+			glong offset;
+
+			offset = g_utf8_pointer_to_offset (title, p2);
+			path = g_strndup (title, offset);
+
+			offset = g_utf8_pointer_to_offset (p2, p);
+			folder = g_strndup (p2, offset);
+
+			ensure_folder (menu, path, folder);
+
+			g_free (folder);
+			g_free (path);
+
+			p2 = p + 1;
+		}
+
+		p = (*p != '\0') ? g_utf8_next_char (p) : NULL;
+	}
+
+	path = g_strdup_printf (BOOKMARKS_MENU_PATH "/%s", title);
+	g_free (title);
+
+	return path;
+}
+
 static void
 ephy_bookmarks_menu_rebuild (EphyBookmarksMenu *menu)
 {
@@ -234,10 +319,10 @@ ephy_bookmarks_menu_rebuild (EphyBookmarksMenu *menu)
 
 	LOG ("Rebuilding bookmarks menu")
 
-	START_PROFILER ("Rebuilding bookmarks menu")
-
 	ephy_bookmarks_menu_clean (menu);
 
+	START_PROFILER ("Rebuilding bookmarks menu")
+
 	topics = ephy_bookmarks_get_keywords (p->bookmarks);
 	not_categorized = ephy_bookmarks_get_not_categorized (p->bookmarks);
 	children = ephy_node_get_children (topics);
@@ -267,41 +352,16 @@ ephy_bookmarks_menu_rebuild (EphyBookmarksMenu *menu)
 
 	for (l = node_list; l != NULL; l = l->next)
 	{
-		char verb[30], name[30], path[60];
-		const char *tmp;
-		char *title;
-		EphyNode *child;
-		GtkAction *action;
+		char *path;
 
-		child = l->data;
-
-		tmp = ephy_node_get_property_string (child, EPHY_NODE_KEYWORD_PROP_NAME);
-		title = ephy_string_double_underscores (tmp);
-		
-		g_sprintf (verb, "OpenTopic%ld", ephy_node_get_id (child));
-		g_sprintf (name, "%sName", verb);
-		g_sprintf (path, "%s/%s", BOOKMARKS_MENU_PATH, name);
-
-		action = g_object_new (GTK_TYPE_ACTION,
-				       "name", verb,
-				       "label", title,
-				       "hide_if_empty", FALSE,
-				       NULL);
-		gtk_action_group_add_action (p->action_group, action);
-		g_object_unref (action);
-		g_free (title);
-
-		gtk_ui_manager_add_ui (p->merge, p->ui_id,
-				       BOOKMARKS_MENU_PATH,
-				       name, verb,
-				       GTK_UI_MANAGER_MENU, FALSE);
-
-		add_bookmarks_menu (menu, child, path);
+		path = create_submenu (menu, l->data);
+		create_menu (menu, l->data, path);
+		g_free (path);
 	}
 
 	if (ephy_node_get_n_children (not_categorized) > 0)
 	{
-		add_bookmarks_menu (menu, not_categorized, BOOKMARKS_MENU_PATH);
+		create_menu (menu, not_categorized, BOOKMARKS_MENU_PATH);
 	}
 
 	g_list_free (node_list);
@@ -427,7 +487,7 @@ topic_child_changed_cb (EphyNode *node,
 		return;
 	}
 
-	g_snprintf (name, sizeof (name), "OpenTopic%ld", ephy_node_get_id (child));
+	g_snprintf (name, sizeof (name), "OpenTopic%d", ephy_node_get_id (child));
 
 	action = gtk_action_group_get_action (menu->priv->action_group, name);
 
