@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-topics-selector.h,v 1.4 2003/09/01 23:20:17 chpe Exp $
+ *  $Id: ephy-topics-selector.h,v 1.6 2004/06/21 18:18:14 marco Exp $
  */
 
 #ifndef EPHY_TOPICS_SELECTOR_H
@@ -40,6 +40,7 @@ typedef struct
 {
 	GtkTreeView parent;
 
+	/*< private >*/
 	EphyTopicsSelectorPrivate *priv;
 } EphyTopicsSelector;
 
@@ -56,6 +57,8 @@ GtkWidget	    *ephy_topics_selector_new             (EphyBookmarks *bookmarks,
 void		     ephy_topics_selector_set_bookmark    (EphyTopicsSelector *selector,
 							   EphyNode *bookmark);
 
+void		     ephy_topics_selector_new_topic	  (EphyTopicsSelector *selector);
+
 void		     ephy_topics_selector_apply		  (EphyTopicsSelector *selector);
 
 
