@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-history-window.c,v 1.68 2004/03/30 22:57:11 piers Exp $
+ *  $Id: ephy-history-window.c,v 1.69 2004/06/21 19:25:41 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -24,6 +24,14 @@
 
 #include <gtk/gtktable.h>
 #include <gtk/gtklabel.h>
+#include <gtk/gtkradioaction.h>
+#include <gtk/gtkmain.h>
+#include <gtk/gtkalignment.h>
+#include <gtk/gtktreeselection.h>
+#include <gtk/gtkhpaned.h>
+#include <gtk/gtkbutton.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtkclipboard.h>
 #include <gtk/gtkstock.h>
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtkhbox.h>
