@@ -1,6 +1,6 @@
 /*
  *  Copyright (C) 2002 Christophe Fergeau
- *  Copyright (C) 2003 Marco Pesenti Gritti
+ *  Copyright (C) 2003, 2004 Marco Pesenti Gritti
  *  Copyright (C) 2003, 2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-notebook.c,v 1.32.4.1 2004/04/02 22:33:03 chpe Exp $
+ *  $Id: ephy-notebook.c,v 1.41 2004/07/05 00:02:09 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,6 +35,8 @@
 #include "ephy-shell.h"
 #include "ephy-debug.h"
 #include "ephy-favicon-cache.h"
+#include "ephy-ellipsizing-label.h"
+#include "ephy-spinner.h"
 
 #include <glib-object.h>
 #include <gtk/gtkeventbox.h>
@@ -51,17 +53,16 @@
 #include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 
+#define TAB_WIDTH_N_CHARS 20
+
 #define AFTER_ALL_TABS -1
 #define NOT_IN_APP_WINDOWS -2
-#define TAB_MIN_SIZE 60
-#define TAB_NB_MAX    8
 
 #define EPHY_NOTEBOOK_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_NOTEBOOK, EphyNotebookPrivate))
 
 struct EphyNotebookPrivate
 {
 	GList *focused_pages;
-	GList *opened_tabs;
 	GtkTooltips *title_tips;
 	guint tabs_vis_notifier_id;
 	gulong motion_notify_handler_id;
@@ -330,33 +331,6 @@ static gint find_notebook_and_tab_at_pos (gint abs_x, gint abs_y,
 	}
 }
 
-static void
-tab_label_set_size (GtkWidget *window, GtkWidget *label)
-{
-	int label_width;
-
-	label_width = window->allocation.width/TAB_NB_MAX;
-
-	if (label_width < TAB_MIN_SIZE) label_width = TAB_MIN_SIZE;
-
-	gtk_widget_set_size_request (label, label_width, -1);
-}
-
-static void
-tab_label_size_request_cb (GtkWidget *window,
-			   GtkRequisition *requisition,
-			   GtkWidget *tab)
-{
-	GtkWidget *hbox;
-	GtkWidget *nb;
-
-	nb = tab->parent;
-
-	hbox = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb),
-					   tab);
-	tab_label_set_size (window, hbox);
-}
-
 void
 ephy_notebook_move_tab (EphyNotebook *src,
 			EphyNotebook *dest,
@@ -378,7 +352,7 @@ ephy_notebook_move_tab (EphyNotebook *src,
 		/* make sure the tab isn't destroyed while we move it */
 		g_object_ref (tab);
 		ephy_notebook_remove_tab (src, tab);
-		ephy_notebook_insert_tab (dest, tab, dest_position, TRUE);
+		ephy_notebook_add_tab (dest, tab, dest_position, TRUE);
 		g_object_unref (tab);
 	}
 }
@@ -438,10 +412,6 @@ move_tab (EphyNotebook *notebook,
 		ephy_notebook_move_tab (EPHY_NOTEBOOK (notebook), NULL,
 					EPHY_TAB (cur_tab),
 					dest_position);
-
-		/* Reset the list of newly opened tabs when moving tabs. */
-		g_list_free (notebook->priv->opened_tabs);
-		notebook->priv->opened_tabs = NULL;
 	}
 }
 
@@ -615,26 +585,15 @@ ephy_notebook_switch_page_cb (GtkNotebook *notebook,
 
 	nb->priv->focused_pages = g_list_append (nb->priv->focused_pages,
 						 child);
-
-	/* Reset the list of newly opened tabs when switching tabs. */
-	g_list_free (nb->priv->opened_tabs);
-	nb->priv->opened_tabs = NULL;
 }
 
-#define INSANE_NUMBER_OF_URLS	20
-
 static void
 notebook_drag_data_received_cb (GtkWidget* widget, GdkDragContext *context,
 				gint x, gint y, GtkSelectionData *selection_data,
 				guint info, guint time, EphyTab *tab)
 {
-	EphyEmbed *embed = NULL;
-	EphyWindow *window = NULL;
-	GtkWidget *toplevel;
-	GList *uris = NULL, *l;
+	GList *uri_list = NULL;
 	GnomeVFSURI *uri;
-	gchar *url = NULL;
-	guint num = 0;
 	gchar **tmp;
 
 	g_signal_stop_emission_by_name (widget, "drag_data_received");
@@ -650,62 +609,31 @@ notebook_drag_data_received_cb (GtkWidget* widget, GdkDragContext *context,
 		if (tmp)
 		{
 			uri = gnome_vfs_uri_new (tmp[0]);
-			if (uri) uris = g_list_append (uris, uri);
+			if (uri)
+			{
+				uri_list = g_list_append (uri_list, uri);
+			}
 			g_strfreev (tmp);
 		}
 	}
 	else if (selection_data->target == gdk_atom_intern (EPHY_DND_URI_LIST_TYPE, FALSE))
 	{
-		uris = gnome_vfs_uri_list_parse (selection_data->data);
+		uri_list = gnome_vfs_uri_list_parse (selection_data->data);
 	}
 	else
 	{
 		g_assert_not_reached ();
 	}
 
-	if (uris == NULL) return;
-
-	toplevel = gtk_widget_get_toplevel (widget);
-	g_return_if_fail (EPHY_IS_WINDOW (toplevel));
-	window = EPHY_WINDOW (toplevel);
-
-	if (tab != NULL)
+	if (uri_list)
 	{
-		embed = ephy_tab_get_embed (tab);
-		g_return_if_fail (EPHY_IS_EMBED (embed));
+		EphyWindow *window;
+
+		window = EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (widget)));
+
+		ephy_window_load_in_tabs (window, tab, uri_list);
+		gnome_vfs_uri_list_free (uri_list);
 	}
-
-	l = uris;
-	while (l != NULL && num < INSANE_NUMBER_OF_URLS)
-	{
-		uri = (GnomeVFSURI*) l->data;
-		url = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
-
-		if (num == 0 && embed != NULL)
-		{
-			/**
-			 * The first url is special: if the drag was to an
-			 * existing tab, load it there
-			 */
-			ephy_embed_load_url (embed, url);
-		}
-		else
-		{
-			tab = ephy_shell_new_tab (ephy_shell, window,
-						  tab, url,
-						  EPHY_NEW_TAB_OPEN_PAGE |
-						  EPHY_NEW_TAB_IN_EXISTING_WINDOW |
-						  (tab ? EPHY_NEW_TAB_APPEND_AFTER :
-							 EPHY_NEW_TAB_APPEND_LAST));
-		}
-
-		g_free (url);
-		url = NULL;
-		l = l->next;
-		++num;
-	}
-
-	gnome_vfs_uri_list_free (uris);
 }
 
 /*
@@ -753,7 +681,6 @@ ephy_notebook_init (EphyNotebook *notebook)
 	notebook->priv->drag_in_progress = FALSE;
 	notebook->priv->motion_notify_handler_id = 0;
 	notebook->priv->focused_pages = NULL;
-	notebook->priv->opened_tabs = NULL;
 	notebook->priv->show_tabs = TRUE;
 
 	g_signal_connect (notebook, "button-press-event",
@@ -791,7 +718,6 @@ ephy_notebook_finalize (GObject *object)
 	{
 		g_list_free (notebook->priv->focused_pages);
 	}
-	g_list_free (notebook->priv->opened_tabs);
 	g_object_unref (notebook->priv->title_tips);
 
 	LOG ("EphyNotebook finalised %p", object)
@@ -802,22 +728,23 @@ ephy_notebook_finalize (GObject *object)
 static void
 sync_load_status (EphyTab *tab, GParamSpec *pspec, GtkWidget *proxy)
 {
-	GtkWidget *animation = NULL, *icon = NULL;
+	GtkWidget *spinner, *icon;
 
-	animation = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "loading-image"));
+	spinner = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "spinner"));
 	icon = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "icon"));
-	g_return_if_fail (animation != NULL && icon != NULL);
+	g_return_if_fail (spinner != NULL && icon != NULL);
 
-	switch (ephy_tab_get_load_status (tab))
+	if (ephy_tab_get_load_status (tab))
 	{
-		case TRUE:
-			gtk_widget_hide (icon);
-			gtk_widget_show (animation);
-			break;
-		case FALSE:
-			gtk_widget_hide (animation);
-			gtk_widget_show (icon);
-			break;
+		gtk_widget_hide (icon);
+		gtk_widget_show (spinner);
+		ephy_spinner_start (EPHY_SPINNER (spinner));
+	}
+	else
+	{
+		ephy_spinner_stop (EPHY_SPINNER (spinner));
+		gtk_widget_hide (spinner);
+		gtk_widget_show (icon);
 	}
 }
 
@@ -867,7 +794,7 @@ sync_label (EphyTab *tab, GParamSpec *pspec, GtkWidget *proxy)
 
 	if (title)
 	{
-		gtk_label_set_label (GTK_LABEL (label), title);
+		ephy_ellipsizing_label_set_text (EPHY_ELLIPSIZING_LABEL (label), title);
 		gtk_tooltips_set_tip (tips, ebox, title, NULL);
 	}
 }
@@ -887,16 +814,38 @@ close_button_clicked_cb (GtkWidget *widget, GtkWidget *tab)
 	}
 }
 
+static void
+tab_label_style_set_cb (GtkWidget *label,
+			GtkStyle *previous_style,
+			GtkWidget *hbox)
+{
+	PangoFontMetrics *metrics;
+	PangoContext *context;
+	int char_width, h, w;
+
+	context = gtk_widget_get_pango_context (label);
+	metrics = pango_context_get_metrics (context,
+			                     label->style->font_desc,
+					     pango_context_get_language (context));
+
+	char_width = pango_font_metrics_get_approximate_char_width (metrics);
+	pango_font_metrics_unref (metrics);
+
+	LOG ("tab_label_size_request_cb label %p char_width %d total %d",
+	     label, char_width, TAB_WIDTH_N_CHARS * PANGO_PIXELS (char_width))
+
+	gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &w, &h);
+
+	gtk_widget_set_size_request
+		(hbox, TAB_WIDTH_N_CHARS * PANGO_PIXELS(char_width) + 2 * w, -1);
+}
+
 static GtkWidget *
 build_tab_label (EphyNotebook *nb, EphyTab *tab)
 {
-	GtkWidget *label, *hbox, *close_button, *image;
-	int h, w;
-	GClosure *closure;
-	GtkWidget *window;
-	GtkWidget *loading_image, *icon;
-	GtkWidget *label_ebox;
-	GdkPixbufAnimation *loading_pixbuf;
+	GtkWidget *window, *hbox, *label_hbox, *label_ebox;
+	GtkWidget *label, *close_button, *image, *spinner, *icon;
+	int h = -1, w = -1;
 
 	window = gtk_widget_get_toplevel (GTK_WIDGET (nb));
 
@@ -906,6 +855,13 @@ build_tab_label (EphyNotebook *nb, EphyTab *tab)
 	 * equal amount of space around the label */
 	hbox = gtk_hbox_new (FALSE, 4);
 
+	label_ebox = gtk_event_box_new ();
+	gtk_event_box_set_visible_window (GTK_EVENT_BOX (label_ebox), FALSE);
+	gtk_box_pack_start (GTK_BOX (hbox), label_ebox, TRUE, TRUE, 0);
+
+	label_hbox = gtk_hbox_new (FALSE, 4);
+	gtk_container_add (GTK_CONTAINER (label_ebox), label_hbox);
+
 	/* setup close button */
 	close_button = gtk_button_new ();
 	gtk_button_set_relief (GTK_BUTTON (close_button),
@@ -913,58 +869,46 @@ build_tab_label (EphyNotebook *nb, EphyTab *tab)
 	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE,
 					  GTK_ICON_SIZE_MENU);
 	gtk_widget_set_size_request (close_button, w, h);
-	gtk_container_add (GTK_CONTAINER (close_button),
-			   image);
-
-	/* setup load feedback image */
-	/* FIXME: make the animation themeable */
-	loading_pixbuf = gdk_pixbuf_animation_new_from_file (ephy_file ("epiphany-tab-loading.gif"), NULL);
-	loading_image = gtk_image_new_from_animation (loading_pixbuf);
-	g_object_unref (loading_pixbuf);
-	gtk_box_pack_start (GTK_BOX (hbox), loading_image, FALSE, FALSE, 0);
-
-	/* setup site icon, empty by default */
-	icon = gtk_image_new ();
-	gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 0);
-
-	/* setup label */
-	label_ebox = gtk_event_box_new ();
-	gtk_event_box_set_visible_window (GTK_EVENT_BOX (label_ebox), FALSE);
-        label = gtk_label_new ("");
-	gtk_misc_set_alignment (GTK_MISC (label), 0.00, 0.5);
-        gtk_misc_set_padding (GTK_MISC (label), 4, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), label_ebox, TRUE, TRUE, 0);
-	gtk_container_add (GTK_CONTAINER (label_ebox), label);
-
-	tab_label_set_size (GTK_WIDGET (window), hbox);
-
-	closure = g_cclosure_new (G_CALLBACK (tab_label_size_request_cb),
-				  tab, NULL);
-	g_object_watch_closure (G_OBJECT (label), closure);
-	g_signal_connect_closure_by_id (G_OBJECT (window),
-		g_signal_lookup ("size_request",
-				 G_OBJECT_TYPE (G_OBJECT (window))), 0,
-                                 closure,
-                                 FALSE);
-
-	/* setup button */
-	gtk_box_pack_start (GTK_BOX (hbox), close_button,
-                            FALSE, FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (close_button), image);
+	gtk_box_pack_start (GTK_BOX (hbox), close_button, FALSE, FALSE, 0);
 
 	g_signal_connect (G_OBJECT (close_button), "clicked",
                           G_CALLBACK (close_button_clicked_cb),
                           tab);
 
+	/* setup load feedback */
+	spinner = ephy_spinner_new ();
+	ephy_spinner_set_size (EPHY_SPINNER (spinner), GTK_ICON_SIZE_MENU);
+	gtk_box_pack_start (GTK_BOX (label_hbox), spinner, FALSE, FALSE, 0);
+
+	/* setup site icon, empty by default */
+	icon = gtk_image_new ();
+	gtk_box_pack_start (GTK_BOX (label_hbox), icon, FALSE, FALSE, 0);
+
+	/* setup label */
+        label = ephy_ellipsizing_label_new ("");
+	ephy_ellipsizing_label_set_mode (EPHY_ELLIPSIZING_LABEL (label),
+					 EPHY_ELLIPSIZE_START);
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+        gtk_misc_set_padding (GTK_MISC (label), 0, 0);
+	gtk_box_pack_start (GTK_BOX (label_hbox), label, TRUE, TRUE, 0);
+
+	/* Set minimal size */
+	g_signal_connect (label, "style-set",
+			  G_CALLBACK (tab_label_style_set_cb), hbox);
+
 	gtk_widget_show (hbox);
 	gtk_widget_show (label_ebox);
+	gtk_widget_show (label_hbox);
 	gtk_widget_show (label);
 	gtk_widget_show (image);
 	gtk_widget_show (close_button);
-
+	
 	g_object_set_data (G_OBJECT (hbox), "label", label);
 	g_object_set_data (G_OBJECT (hbox), "label-ebox", label_ebox);
-	g_object_set_data (G_OBJECT (hbox), "loading-image", loading_image);
+	g_object_set_data (G_OBJECT (hbox), "spinner", spinner);
 	g_object_set_data (G_OBJECT (hbox), "icon", icon);
+	g_object_set_data (G_OBJECT (hbox), "close-button", close_button);
 	g_object_set_data (G_OBJECT (hbox), "tooltips", nb->priv->title_tips);
 
 	return hbox;
@@ -979,10 +923,10 @@ ephy_notebook_set_show_tabs (EphyNotebook *nb, gboolean show_tabs)
 }
 
 void
-ephy_notebook_insert_tab (EphyNotebook *nb,
-			  EphyTab *tab,
-			  int position,
-			  gboolean jump_to)
+ephy_notebook_add_tab (EphyNotebook *nb,
+		       EphyTab *tab,
+		       int position,
+		       gboolean jump_to)
 {
 	GtkWidget *label;
 
@@ -992,29 +936,6 @@ ephy_notebook_insert_tab (EphyNotebook *nb,
 
 	update_tabs_visibility (nb, TRUE);
 
-	if (position == EPHY_NOTEBOOK_INSERT_GROUPED)
-	{
-		/* Keep a list of newly opened tabs, if the list is empty open the new
-		 * tab after the current one. If it's not, add it after the newly
-		 * opened tabs.
-		 */
-		if (nb->priv->opened_tabs != NULL)
-		{
-			GList *last = g_list_last (nb->priv->opened_tabs);
-			GtkWidget *last_tab = last->data;
-			position = gtk_notebook_page_num
-				    (GTK_NOTEBOOK (nb), last_tab) + 1;
-		}
-		else
-		{
-			position = gtk_notebook_get_current_page
-				    (GTK_NOTEBOOK (nb)) + 1;
-		}
-		nb->priv->opened_tabs =
-			g_list_append (nb->priv->opened_tabs, tab);
-	}
-
-
 	gtk_notebook_insert_page (GTK_NOTEBOOK (nb), GTK_WIDGET (tab),
 				  label, position);
 
@@ -1091,6 +1012,8 @@ ephy_notebook_remove_tab (EphyNotebook *nb,
 	{
 		GtkWidget *window;
 		window = gtk_widget_get_toplevel (GTK_WIDGET (nb));
+
+		g_signal_emit (G_OBJECT (nb), signals[TAB_REMOVED], 0, tab);
 		gtk_widget_destroy (window);
 		return;
 	}
@@ -1098,7 +1021,6 @@ ephy_notebook_remove_tab (EphyNotebook *nb,
 	/* Remove the page from the focused pages list */
 	nb->priv->focused_pages =  g_list_remove (nb->priv->focused_pages,
 						  tab);
-	nb->priv->opened_tabs = g_list_remove (nb->priv->opened_tabs, tab);
 
 	position = gtk_notebook_page_num (GTK_NOTEBOOK (nb), GTK_WIDGET (tab));
 	curr = gtk_notebook_get_current_page (GTK_NOTEBOOK (nb));
