@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 2000-2003 Marco Pesenti Gritti
+ *  Copyright (C) 2000-2004 Marco Pesenti Gritti
+ *  Copyright (C) 2003, 2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-shell.c,v 1.85 2004/03/29 18:48:38 chpe Exp $
+ *  $Id: ephy-shell.c,v 1.88 2004/06/19 17:21:19 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -39,6 +40,7 @@
 #include "toolbar.h"
 #include "ephy-session.h"
 #include "downloader-view.h"
+#include "egg-toolbars-model.h"
 #include "ephy-toolbars-model.h"
 #include "ephy-automation.h"
 #include "print-dialog.h"
@@ -63,7 +65,7 @@ struct EphyShellPrivate
 	BonoboGenericFactory *automation_factory;
 	EphySession *session;
 	EphyBookmarks *bookmarks;
-	EphyToolbarsModel *toolbars_model;
+	EggToolbarsModel *toolbars_model;
 	EggToolbarsModel *fs_toolbars_model;
 	EphyExtensionsManager *extensions_manager;
 	GtkWidget *bme;
@@ -544,7 +546,6 @@ ephy_shell_new_tab (EphyShell *shell,
 	EphyTab *tab;
 	EphyEmbed *embed;
 	gboolean in_new_window = TRUE;
-	gboolean grouped;
 	gboolean jump_to;
 	EphyEmbed *previous_embed = NULL;
 	GtkWidget *nb;
@@ -572,10 +573,6 @@ ephy_shell_new_tab (EphyShell *shell,
 		previous_embed = ephy_tab_get_embed (previous_tab);
 	}
 
-	grouped = ((flags & EPHY_NEW_TAB_OPEN_PAGE ||
-		    flags & EPHY_NEW_TAB_APPEND_GROUPED)) &&
-		  !(flags & EPHY_NEW_TAB_APPEND_LAST);
-
 	if ((flags & EPHY_NEW_TAB_APPEND_AFTER) && previous_embed != NULL)
 	{
 		nb = ephy_window_get_notebook (window);
@@ -584,7 +581,7 @@ ephy_shell_new_tab (EphyShell *shell,
 	}
 	else
 	{
-		position = grouped ? EPHY_NOTEBOOK_INSERT_GROUPED : EPHY_NOTEBOOK_INSERT_LAST;
+		position = EPHY_NOTEBOOK_ADD_LAST;
 	}
 
 	tab = ephy_tab_new ();
@@ -664,11 +661,16 @@ ephy_shell_get_toolbars_model (EphyShell *shell, gboolean fullscreen)
 	{
 		if (shell->priv->fs_toolbars_model == NULL)
 		{
+			gboolean success;
 			const char *xml;
 
 			shell->priv->fs_toolbars_model = egg_toolbars_model_new ();
 			xml = ephy_file ("epiphany-fs-toolbar.xml");
-			egg_toolbars_model_load (shell->priv->fs_toolbars_model, xml);
+			g_return_val_if_fail (xml != NULL, NULL);
+
+			success = egg_toolbars_model_load
+				(shell->priv->fs_toolbars_model, xml);
+			g_return_val_if_fail (success, NULL);
 		}
 
 		return G_OBJECT (shell->priv->fs_toolbars_model);
@@ -678,16 +680,22 @@ ephy_shell_get_toolbars_model (EphyShell *shell, gboolean fullscreen)
 		if (shell->priv->toolbars_model == NULL)
 		{
 			EphyBookmarks *bookmarks;
+			EggToolbarsModel *bookmarksbar_model;
 
+			shell->priv->toolbars_model = ephy_toolbars_model_new ();
+
+			/* get the bookmarks toolbars model. we have to do this
+			 * before loading the toolbars model from disk, since
+			 * this will connect the get_item_* signals
+			 */
 			bookmarks = ephy_shell_get_bookmarks (shell);
+			bookmarksbar_model = ephy_bookmarks_get_toolbars_model (bookmarks);
 
-			shell->priv->toolbars_model = ephy_toolbars_model_new (bookmarks);
-
-			g_object_set (bookmarks, "toolbars_model",
-				      shell->priv->toolbars_model, NULL);
+			/* ok, now we can load the model */
+			ephy_toolbars_model_load
+			(EPHY_TOOLBARS_MODEL (shell->priv->toolbars_model));
 		}
 
-
 		return G_OBJECT (shell->priv->toolbars_model);
 	}
 }
@@ -699,12 +707,19 @@ ephy_shell_get_extensions_manager (EphyShell *es)
 
 	if (es->priv->extensions_manager == NULL)
 	{
+		char *path;
+
 		/* Instantiate extensions manager */
 		es->priv->extensions_manager = ephy_extensions_manager_new ();
 
 		/* load the extensions */
 		ephy_extensions_manager_load_dir (es->priv->extensions_manager,
 						  EXTENSIONS_DIR);
+
+		path = g_build_filename (ephy_dot_dir (), "extensions", NULL);
+		ephy_extensions_manager_load_dir (es->priv->extensions_manager,
+						  path);
+		g_free (path);
 	}
 
 	return G_OBJECT (es->priv->extensions_manager);
