@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: prefs-dialog.c,v 1.80 2004/05/08 20:11:17 chpe Exp $
+ *  $Id: prefs-dialog.c,v 1.82 2004/06/21 19:25:41 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -44,6 +44,9 @@
 #include "ephy-tree-model-sort.h"
 
 #include <glib/gi18n.h>
+#include <gtk/gtkcellrenderertext.h>
+#include <gtk/gtkcelllayout.h>
+#include <gtk/gtktreeselection.h>
 #include <gtk/gtkframe.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkvbox.h>
@@ -63,8 +66,6 @@ static void prefs_dialog_class_init	(PrefsDialogClass *klass);
 static void prefs_dialog_init		(PrefsDialog *pd);
 
 /* Glade callbacks */
-void prefs_proxy_auto_url_reload_cb		(GtkWidget *button,
-						 EphyDialog *dialog);
 void prefs_clear_cache_button_clicked_cb	(GtkWidget *button,
 						 gpointer data);
 void prefs_dialog_response_cb			(GtkDialog *widget,
